/* GPS Logger */
/* ukad ATmega 8MHz */

/* F_CPU w Makefile
#define F_CPU 8000000L 
#define F_CPU 11059200UL*/
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>
#include <util/delay.h>
#include <string.h>
#include <stdlib.h>

#include "led_display.h"

#include "init.c"

/*
  Fuse: BE D1
*/

void beep(void);

void init_ports(void);
void init_overflows(void);

void show_digit(int);
void show_time(int);

void start_liczenia(void);
void stop_liczenia(void);

#define P1_pressed	!(PINB & _BV(4))
#define P2_pressed	!(PINB & _BV(3))
#define P3_pressed	!(PINB & _BV(1))
#define LED_toggle	PORTB ^= _BV(0)
#define LED_on 		PORTB |= _BV(0)
#define LED_off 	PORTB &= ~_BV(0)


extern volatile struct disp_t disp;

volatile int napiecie;
volatile int timeleft = 20;	// aktualnie liczona warto	(przy wczeniu 20 minut)
volatile int timeset  = 20;	// poprzednio ustawiona warto
#define TRYB_USTAW	0
#define TRYB_LICZY	1
#define TRYB_PIKA	2
volatile char tryb = 0;
volatile int sekundy = 0;
#define POKAZ_MIN	0
#define POKAZ_SEK	1
volatile int pokaz=POKAZ_MIN;

int main(void)
{
	//Wy
	DDRC  = 0xFE;	//output
	PORTC = 0x00;	//on
	/*DDRC  |=  _BV(2);	//output
	PORTC |=  _BV(2);	//on
	DDRC  |=  _BV(3);	//output
	PORTC |=  _BV(3);	//on
	DDRC  |=  _BV(4);	//output
	PORTC |=  _BV(4);	//on
	DDRC  |=  _BV(5);	//output
	PORTC |=  _BV(5);	//on
	*/
	//We
    DDRD  = 0xFF;	//output
	PORTD =	0xFF;	//off
	
	init_ports();
	init_display();
	init_overflows();
	
	_delay_ms(100);
	
	show_time(timeleft);
	
	while(1){
		
		// Przycisk 1
		if( P1_pressed && tryb == TRYB_USTAW ){
			
			if( timeleft-1 > 1 )
				timeleft--;
			else
				timeleft = 1;
			
			show_time(timeleft);
			_delay_ms(80);
			
			for( int i=0; i<=50; i++ ){
				
				if( !P1_pressed ) break;
				_delay_ms(10);
				
				if( P2_pressed && P3_pressed ){
					show_digit(napiecie);
					disp.kropka[1]=1;
					_delay_ms(1000);
					disp.kropka[1]=0;
					show_time(timeleft);
				}
			}
			
			while( P1_pressed ){
				
				for( int i=0; i<=50; i++ ){
					
					if( !P1_pressed ) break;
					_delay_ms(10);
					
				}
				
				if( P1_pressed ){
					if( timeleft-10 > 0 )
						timeleft -= 10;
					else
						timeleft = 1;
					
					show_time(timeleft);
				}
			}
			_delay_ms(80);
			
		}
		else if( P1_pressed && tryb == TRYB_LICZY ){
			
			pokaz = POKAZ_MIN;
			show_time( timeleft );
			
			_delay_ms(80);
			while( P1_pressed ){}
			_delay_ms(80);
			
		}
		
		
		// Przycisk 2
		if( P2_pressed && tryb == TRYB_USTAW ){
			
			if( timeleft+1 <= 720 )
				timeleft++;
			
			show_time(timeleft);
			_delay_ms(80);
			
			for( int i=0; i<=50; i++ ){
				
				if(!P2_pressed) break;
				_delay_ms(10);
			}
			
			while( P2_pressed ){
				
				for( int i=0; i<=50; i++ ){
					
					if(!P2_pressed) break;
					_delay_ms(10);
					
				}
				
				if( P2_pressed ){
					if( timeleft+10 <= 720 )
						timeleft+=10;
					else
						timeleft = 720;
					
					show_time(timeleft);
				}
			}
			_delay_ms(80);
			
		}
		else if( P2_pressed && tryb == TRYB_LICZY ){
		
			pokaz = POKAZ_SEK;
			show_digit( sekundy );
			
			_delay_ms(80);
			while( P2_pressed ){}
			_delay_ms(80);
			
		}
		
		// Przycisk 3
		if( P3_pressed ){
			
			if( tryb == TRYB_USTAW )
				start_liczenia();
			else
				stop_liczenia();
			
			_delay_ms(80);
			while( P3_pressed ){}
			_delay_ms(80);
			
		}
		
		//if( tryb == TRYB_LICZY ){
			
		//}
		
		if( tryb == TRYB_PIKA ){
			
			disp.znak[0]=10;
			disp.znak[1]=10;
			disp.znak[2]=10;
			disp.znak[3]=10;
			
			while( 1 ){
				
				disp.znak[1]= ( disp.znak[1]==10 ? 8 : 10 );
				disp.znak[2]= disp.znak[1];
				
				if( P1_pressed || P2_pressed || P3_pressed ) break;
				PORTB &= ~_BV(2);	//on
				_delay_ms(100);
				
				disp.znak[1]= ( disp.znak[1]==10 ? 8 : 10 );
				disp.znak[2]= disp.znak[1];
				
				PORTB |=  _BV(2);	//off
				if( P1_pressed || P2_pressed || P3_pressed ) break;
				_delay_ms(50);
				
				disp.znak[1]= ( disp.znak[1]==10 ? 8 : 10 );
				disp.znak[2]= disp.znak[1];
				
				if( P1_pressed || P2_pressed || P3_pressed ) break;
				PORTB &= ~_BV(2);	//on
				_delay_ms(100);
				
				disp.znak[1]= ( disp.znak[1]==10 ? 8 : 10 );
				disp.znak[2]= disp.znak[1];
				
				PORTB |=  _BV(2);	//off
				if( P1_pressed || P2_pressed || P3_pressed ) break;
				_delay_ms(250);
				
			}
			
			PORTB |=  _BV(2);	//off
			show_time(timeleft);
			_delay_ms(80);
			while( P1_pressed || P2_pressed || P3_pressed );
			_delay_ms(80);
			tryb = TRYB_USTAW;
		}
	}
}

void beep(void)
{
	PORTB &= ~_BV(2);	//on
	_delay_ms(100);
	PORTB |=  _BV(2);	//off
	_delay_ms(50);
	PORTB &= ~_BV(2);	//on
	_delay_ms(100);
	PORTB |=  _BV(2);	//off
}

SIGNAL(SIG_OVERFLOW2){

	disp.kropka[0]=0;
	disp.kropka[1]=0;
	disp.kropka[2]=0;
	disp.kropka[3]=0;
	
	sekundy--;
	if( sekundy <= 0 ){
		
		sekundy = 59;
		
		
		if( timeleft == 0 ){
			
			stop_liczenia();
			
			tryb = TRYB_PIKA;
			
		}
		else{
			
			timeleft--;
			
			if(timeleft == 0){
				pokaz = POKAZ_SEK;
				show_digit(sekundy);
			}else{
				pokaz = POKAZ_MIN;
				show_time(timeleft);
			}
			
		}
		
	}else if( pokaz == POKAZ_SEK ){
		
		show_digit( sekundy );
		
	}else{
		
		disp.dwukropek = (disp.dwukropek==0?1:0);
		
	}
	
	/*
	if( tryb == TRYB_LICZY ){
		tmp++;
		if(tmp>3){
			tmp=0;
		
		}
		disp.kropka[tmp]=1;
	}
	*/
	// przy okazji sprawd napicie
	ADCSR|= _BV(ADSC);	// wczenie przetwarzania ADC
}

SIGNAL(SIG_OVERFLOW1){

	display();
	TCNT1 = 63136;
	
}

SIGNAL(SIG_ADC){						// przerwanie z przetwornika ADC 

	napiecie = ADCW/1.95;				// obliczenie napicia (400 = 4.00V)
	
	if(napiecie<350) LED_on;			// przy <3.5V zapal diod
	else LED_off;

}


void start_liczenia( void ){

	if( tryb == TRYB_USTAW ){
		
		timeset = timeleft;
		tryb = TRYB_LICZY;
		sekundy = 0;
		show_time(timeleft);
		TIMSK |= _BV(TOIE2);	// wczenie przerwania zegarkowego
		
	}
	
}

void stop_liczenia( void ){
	
	if( tryb == TRYB_LICZY){
		
		tryb = TRYB_USTAW;
		timeleft = timeset;
		TIMSK &= ~_BV(TOIE2);	// wyczenie przerwania zegarkowego
		show_time(timeleft);
		
	}
	
}